@@ -1,4 +1,4 @@
-use std::cmp::max;
+use std::cmp::{max, min};
 use std::collections::BTreeSet;
 use std::convert::TryInto;
 
@@ -6,7 +6,7 @@ use std::convert::TryInto;
 use cosmwasm_std::entry_point;
 use cosmwasm_std::{
     to_binary, Addr, Binary, BlockInfo, Decimal, Deps, DepsMut, Env, MessageInfo, Order, Reply,
-    StdResult, Timestamp, WasmMsg,
+    StdError, StdResult, Timestamp, WasmMsg,
 };
 
 use cw2::set_contract_version;
@@ -19,12 +19,12 @@ use tg_bindings::{
     request_privileges, Ed25519Pubkey, Evidence, EvidenceType, Privilege, PrivilegeChangeMsg,
     Pubkey, TgradeMsg, TgradeSudoMsg, ValidatorDiff, ValidatorUpdate,
 };
-use tg_utils::{JailingDuration, SlashMsg, ADMIN};
+use tg_utils::{ensure_from_older_version, JailingDuration, SlashMsg, ADMIN};
 
 use crate::error::ContractError;
 use crate::msg::{
     EpochResponse, ExecuteMsg, InstantiateMsg, InstantiateResponse, JailingPeriod,
-    ListActiveValidatorsResponse, ListValidatorResponse, ListValidatorSlashingResponse,
+    ListActiveValidatorsResponse, ListValidatorResponse, ListValidatorSlashingResponse, MigrateMsg,
     OperatorResponse, QueryMsg, RewardsDistribution, RewardsInstantiateMsg, ValidatorMetadata,
     ValidatorResponse,
 };
@@ -59,13 +59,13 @@ pub fn instantiate(
     msg.validate()?;
     let membership = Tg4Contract(deps.api.addr_validate(&msg.membership)?);
     membership
-        .total_weight(&deps.querier)
+        .total_points(&deps.querier)
         .map_err(|_| ContractError::InvalidTg4Contract {})?;
     let distribution_contracts = msg.distribution_contracts.validate(deps.api)?;
 
     let cfg = Config {
         membership,
-        min_weight: msg.min_weight,
+        min_points: msg.min_points,
         max_validators: msg.max_validators,
         scaling: msg.scaling,
         epoch_reward: msg.epoch_reward,
@@ -74,7 +74,7 @@ pub fn instantiate(
         double_sign_slash_ratio: msg.double_sign_slash_ratio,
         distribution_contracts,
         // Will be overwritten in reply for rewards contract instantiation
-        rewards_contract: Addr::unchecked(""),
+        validator_group: Addr::unchecked(""),
     };
     CONFIG.save(deps.storage, &cfg)?;
 
@@ -91,9 +91,11 @@ pub fn instantiate(
     for op in msg.initial_keys.into_iter() {
         let oper = deps.api.addr_validate(&op.operator)?;
         let pubkey: Ed25519Pubkey = op.validator_pubkey.try_into()?;
+        op.metadata.validate()?;
         let info = OperatorInfo {
             pubkey,
             metadata: op.metadata,
+            active_validator: false,
         };
         operators().save(deps.storage, &oper, &info)?;
     }
@@ -111,7 +113,7 @@ pub fn instantiate(
 
     let instantiate_rewards_msg = WasmMsg::Instantiate {
         admin: msg.admin,
-        code_id: msg.rewards_code_id,
+        code_id: msg.validator_group_code_id,
         msg: to_binary(&rewards_init)?,
         funds: vec![],
         label: format!("rewards_distribution_{}", env.contract.address),
@@ -151,6 +153,11 @@ pub fn execute(
             info,
             admin.map(|admin| api.addr_validate(&admin)).transpose()?,
         )?),
+        ExecuteMsg::UpdateConfig {
+            min_points,
+            max_validators,
+        } => execute_update_config(deps, info, min_points, max_validators),
+
         ExecuteMsg::RegisterValidatorKey { pubkey, metadata } => {
             execute_register_validator_key(deps, env, info, pubkey, metadata)
         }
@@ -160,9 +167,38 @@ pub fn execute(
         }
         ExecuteMsg::Unjail { operator } => execute_unjail(deps, env, info, operator),
         ExecuteMsg::Slash { addr, portion } => execute_slash(deps, env, info, addr, portion),
+        #[cfg(debug_assertions)]
+        ExecuteMsg::SimulateValidatorSet { validators } => {
+            execute_simulate_validators(deps, info, validators)
+        }
     }
 }
 
+fn execute_update_config(
+    deps: DepsMut,
+    info: MessageInfo,
+    min_points: Option<u64>,
+    max_validators: Option<u32>,
+) -> Result<Response, ContractError> {
+    ADMIN.assert_admin(deps.as_ref(), &info.sender)?;
+
+    CONFIG.update::<_, StdError>(deps.storage, |mut cfg| {
+        if let Some(min_points) = min_points {
+            cfg.min_points = min_points;
+        }
+        if let Some(max_validators) = max_validators {
+            cfg.max_validators = max_validators;
+        }
+        Ok(cfg)
+    })?;
+
+    let res = Response::new()
+        .add_attribute("action", "update_config")
+        .add_attribute("operator", &info.sender);
+
+    Ok(res)
+}
+
 fn execute_register_validator_key(
     deps: DepsMut,
     _env: Env,
@@ -170,10 +206,16 @@ fn execute_register_validator_key(
     pubkey: Pubkey,
     metadata: ValidatorMetadata,
 ) -> Result<Response, ContractError> {
+    metadata.validate()?;
+
     let pubkey: Ed25519Pubkey = pubkey.try_into()?;
     let moniker = metadata.moniker.clone();
 
-    let operator = OperatorInfo { pubkey, metadata };
+    let operator = OperatorInfo {
+        pubkey,
+        metadata,
+        active_validator: false,
+    };
     match operators().may_load(deps.storage, &info.sender)? {
         Some(_) => return Err(ContractError::OperatorRegistered {}),
         None => operators().save(deps.storage, &info.sender, &operator)?,
@@ -272,8 +314,8 @@ fn execute_unjail(
         Ok(Some(expiration)) if (expiration.is_expired(&env.block) || is_admin) => {
             JAIL.remove(deps.storage, operator);
         }
-        // Jail not expired and called by non-admin
-        _ => return Err(AdminError::NotAdmin {}.into()),
+        // Jail not expired
+        _ => return Err(ContractError::JailDidNotExpire {}),
     }
 
     let res = Response::new()
@@ -332,6 +374,29 @@ fn execute_slash(
     Ok(resp)
 }
 
+#[cfg(debug_assertions)]
+fn execute_simulate_validators(
+    deps: DepsMut,
+    _info: MessageInfo,
+    validators: Vec<ValidatorInfo>,
+) -> Result<Response, ContractError> {
+    // Assert admin is not found (contract not instantiated!) for this to be possible
+    let admin_res = ADMIN.get(deps.as_ref());
+    if let Err(err) = admin_res {
+        let not_found = matches!(err, StdError::NotFound { .. });
+        if !not_found {
+            return Err(ContractError::AdminError(AdminError::NotAdmin {}));
+        }
+    } else {
+        return Err(ContractError::AdminError(AdminError::NotAdmin {}));
+    }
+
+    // Store validators
+    VALIDATORS.save(deps.storage, &validators)?;
+
+    Ok(Response::new())
+}
+
 #[cfg_attr(not(feature = "library"), entry_point)]
 pub fn query(deps: Deps, env: Env, msg: QueryMsg) -> Result<Binary, ContractError> {
     use QueryMsg::*;
@@ -345,7 +410,17 @@ pub fn query(deps: Deps, env: Env, msg: QueryMsg) -> Result<Binary, ContractErro
             start_after,
             limit,
         )?)?),
-        ListActiveValidators {} => Ok(to_binary(&list_active_validators(deps, env)?)?),
+        ListActiveValidators { start_after, limit } => Ok(to_binary(&list_active_validators(
+            deps,
+            start_after,
+            limit,
+        )?)?),
+        ListJailedValidators { start_after, limit } => Ok(to_binary(&list_jailed_validators(
+            deps,
+            env,
+            start_after,
+            limit,
+        )?)?),
         SimulateActiveValidators {} => Ok(to_binary(&simulate_active_validators(deps, env)?)?),
         ListValidatorSlashing { operator } => {
             Ok(to_binary(&list_validator_slashing(deps, env, operator)?)?)
@@ -392,8 +467,8 @@ fn query_validator_key(
 }
 
 // settings for pagination
-const MAX_LIMIT: u32 = 30;
-const DEFAULT_LIMIT: u32 = 10;
+const MAX_LIMIT: u32 = 100;
+const DEFAULT_LIMIT: u32 = 30;
 
 fn list_validator_keys(
     deps: Deps,
@@ -420,6 +495,7 @@ fn list_validator_keys(
                 metadata: info.metadata,
                 pubkey: info.pubkey.into(),
                 jailed_until,
+                active_validator: info.active_validator,
             })
         })
         .take(limit)
@@ -432,10 +508,68 @@ fn list_validator_keys(
 
 fn list_active_validators(
     deps: Deps,
-    _env: Env,
+    start_after: Option<String>,
+    limit: Option<u32>,
 ) -> Result<ListActiveValidatorsResponse, ContractError> {
+    let limit = limit.unwrap_or(DEFAULT_LIMIT).min(MAX_LIMIT) as usize;
+    let start_after = maybe_addr(deps.api, start_after)?;
+
     let validators = VALIDATORS.load(deps.storage)?;
-    Ok(ListActiveValidatorsResponse { validators })
+    // Simulate a range query
+    let mut i = 0;
+    if let Some(start_after) = start_after {
+        for v in &validators {
+            if v.operator == start_after {
+                i += 1;
+                break;
+            }
+            i += 1;
+        }
+    }
+    let validators = &validators[i..min(i + limit, validators.len())];
+    Ok(ListActiveValidatorsResponse {
+        validators: Vec::from(validators),
+    })
+}
+
+fn list_jailed_validators(
+    deps: Deps,
+    env: Env,
+    start_after: Option<String>,
+    limit: Option<u32>,
+) -> Result<ListValidatorResponse, ContractError> {
+    let cfg = CONFIG.load(deps.storage)?;
+    let limit = limit.unwrap_or(DEFAULT_LIMIT).min(MAX_LIMIT) as usize;
+    let start_after = maybe_addr(deps.api, start_after)?;
+    let start = start_after.map(|addr| Bound::exclusive(addr.as_str()));
+
+    let validators = JAIL
+        .range(deps.storage, start, None, Order::Ascending)
+        .map(|jail| {
+            let (addr, jailing_period) = jail?;
+            if !(cfg.auto_unjail && jailing_period.is_expired(&env.block)) {
+                Ok(Some((addr, jailing_period)))
+            } else {
+                Ok(None)
+            }
+        })
+        .collect::<Result<Vec<Option<_>>, ContractError>>()?
+        .into_iter()
+        .flatten()
+        .map(|(addr, jailing_period)| {
+            let info = operators().load(deps.storage, &Addr::unchecked(&addr))?;
+            Ok(OperatorResponse {
+                operator: addr.into(),
+                metadata: info.metadata,
+                pubkey: info.pubkey.into(),
+                jailed_until: Some(jailing_period),
+                active_validator: info.active_validator,
+            })
+        })
+        .take(limit)
+        .collect::<Result<Vec<OperatorResponse>, ContractError>>()?;
+
+    Ok(ListValidatorResponse { validators })
 }
 
 fn simulate_active_validators(
@@ -538,9 +672,30 @@ fn end_block(deps: DepsMut, env: Env) -> Result<Response, ContractError> {
     }
 
     let old_validators = VALIDATORS.load(deps.storage)?;
+
     VALIDATORS.save(deps.storage, &validators)?;
     // determine the diff to send back to tendermint
-    let (diff, update_members) = calculate_diff(validators, old_validators);
+    let (diff, add, remove) = calculate_diff(validators, old_validators);
+    let update_members = RewardsDistribution::UpdateMembers {
+        add: add.clone(),
+        remove: remove.clone(),
+    };
+
+    // update operators list with info about whether or not they're active validators
+    for op in add {
+        operators().update::<_, StdError>(deps.storage, &Addr::unchecked(op.addr), |op| {
+            let mut op = op.ok_or_else(|| StdError::generic_err("operator doesn't exist"))?;
+            op.active_validator = true;
+            Ok(op)
+        })?;
+    }
+    for op in remove {
+        operators().update::<_, StdError>(deps.storage, &Addr::unchecked(op), |op| {
+            let mut op = op.ok_or_else(|| StdError::generic_err("operator doesn't exist"))?;
+            op.active_validator = false;
+            Ok(op)
+        })?;
+    }
 
     // Store starting heights of new validators
     match &update_members {
@@ -569,7 +724,7 @@ fn end_block(deps: DepsMut, env: Env) -> Result<Response, ContractError> {
     };
 
     let res = res.add_submessage(SubMsg::new(WasmMsg::Execute {
-        contract_addr: cfg.rewards_contract.to_string(),
+        contract_addr: cfg.validator_group.to_string(),
         msg: to_binary(&update_members)?,
         funds: vec![],
     }));
@@ -587,14 +742,14 @@ fn calculate_validators(
 ) -> Result<(Vec<ValidatorInfo>, Vec<Addr>), ContractError> {
     let cfg = CONFIG.load(deps.storage)?;
 
-    let min_weight = max(cfg.min_weight, 1);
+    let min_points = max(cfg.min_points, 1);
     let scaling: u64 = cfg.scaling.unwrap_or(1).into();
 
     // get all validators from the contract, filtered
     let mut validators = vec![];
     let mut batch = cfg
         .membership
-        .list_members_by_weight(&deps.querier, None, QUERY_LIMIT)?;
+        .list_members_by_points(&deps.querier, None, QUERY_LIMIT)?;
     let mut auto_unjail = vec![];
 
     while !batch.is_empty() && validators.len() < cfg.max_validators as usize {
@@ -602,7 +757,7 @@ fn calculate_validators(
 
         let filtered: Vec<_> = batch
             .into_iter()
-            .filter(|m| m.weight >= min_weight)
+            .filter(|m| m.points >= min_points)
             .filter_map(|m| -> Option<StdResult<_>> {
                 // why do we allow Addr::unchecked here?
                 // all valid keys for `operators()` are already validated before insertion
@@ -621,7 +776,7 @@ fn calculate_validators(
                     Err(err) => return Some(Err(err)),
                     // address not jailed, proceed
                     Ok(None) => (),
-                    // address jailed, but period axpired and auto unjailing enabled, add to
+                    // address jailed, but period expired and auto unjailing enabled, add to
                     // auto_unjail list
                     Ok(Some(expires)) if cfg.auto_unjail && expires.is_expired(&env.block) => {
                         auto_unjail.push(m_addr.clone())
@@ -634,8 +789,7 @@ fn calculate_validators(
                     Ok(ValidatorInfo {
                         operator: m_addr,
                         validator_pubkey: op.pubkey.into(),
-                        metadata: op.metadata,
-                        power: m.weight * scaling,
+                        power: m.points * scaling,
                     })
                 })
             })
@@ -646,7 +800,7 @@ fn calculate_validators(
         // and get the next page
         batch = cfg
             .membership
-            .list_members_by_weight(&deps.querier, last, QUERY_LIMIT)?;
+            .list_members_by_points(&deps.querier, last, QUERY_LIMIT)?;
     }
 
     Ok((validators, auto_unjail))
@@ -669,7 +823,7 @@ fn calculate_validators(
 fn calculate_diff(
     cur_vals: Vec<ValidatorInfo>,
     old_vals: Vec<ValidatorInfo>,
-) -> (ValidatorDiff, RewardsDistribution) {
+) -> (ValidatorDiff, Vec<Member>, Vec<String>) {
     // Compute additions and updates
     let cur: BTreeSet<_> = cur_vals.iter().collect();
     let old: BTreeSet<_> = old_vals.iter().collect();
@@ -682,7 +836,7 @@ fn calculate_diff(
             };
             let member = Member {
                 addr: vi.operator.to_string(),
-                weight: vi.power,
+                points: vi.power,
             };
 
             (update, member)
@@ -715,10 +869,24 @@ fn calculate_diff(
     // Compute, map and append removals to diffs
     diffs.extend(removed_diff);
 
-    (
-        ValidatorDiff { diffs },
-        RewardsDistribution::UpdateMembers { add, remove },
-    )
+    (ValidatorDiff { diffs }, add, remove)
+}
+
+#[cfg_attr(not(feature = "library"), entry_point)]
+pub fn migrate(deps: DepsMut, _env: Env, msg: MigrateMsg) -> Result<Response, ContractError> {
+    ensure_from_older_version(deps.storage, CONTRACT_NAME, CONTRACT_VERSION)?;
+
+    CONFIG.update::<_, StdError>(deps.storage, |mut cfg| {
+        if let Some(min_points) = msg.min_points {
+            cfg.min_points = min_points;
+        }
+        if let Some(max_validators) = msg.max_validators {
+            cfg.max_validators = max_validators;
+        }
+        Ok(cfg)
+    })?;
+
+    Ok(Response::new())
 }
 
 mod evidence {
@@ -844,12 +1012,12 @@ pub fn rewards_instantiate_reply(
 
     let addr = deps.api.addr_validate(&res.contract_address)?;
     CONFIG.update(deps.storage, |mut config| -> StdResult<_> {
-        config.rewards_contract = addr.clone();
+        config.validator_group = addr.clone();
         Ok(config)
     })?;
 
     let data = InstantiateResponse {
-        rewards_contract: addr,
+        validator_group: addr,
     };
 
     let resp = Response::new().set_data(to_binary(&data)?);
@@ -877,16 +1045,14 @@ mod test {
         vals
     }
 
-    fn update_members_msg(remove: Vec<&str>, add: Vec<(&str, u64)>) -> RewardsDistribution {
-        let remove = remove.into_iter().map(str::to_owned).collect();
-        let add = add
+    fn members(members: Vec<(&str, u64)>) -> Vec<Member> {
+        members
             .into_iter()
-            .map(|(addr, weight)| Member {
+            .map(|(addr, points)| Member {
                 addr: addr.to_owned(),
-                weight,
+                points,
             })
-            .collect();
-        RewardsDistribution::UpdateMembers { add, remove }
+            .collect()
     }
 
     // Unit tests for calculate_diff()
@@ -899,36 +1065,22 @@ mod test {
             ValidatorInfo {
                 operator: Addr::unchecked("op1"),
                 validator_pubkey: Pubkey::Ed25519(b"pubkey1".into()),
-                metadata: ValidatorMetadata {
-                    moniker: "moniker1".to_owned(),
-                    identity: None,
-                    website: None,
-                    security_contact: None,
-                    details: None,
-                },
                 power: 1,
             },
             ValidatorInfo {
                 operator: Addr::unchecked("op2"),
                 validator_pubkey: Pubkey::Ed25519(b"pubkey2".into()),
-                metadata: ValidatorMetadata {
-                    moniker: "moniker2".to_owned(),
-                    identity: None,
-                    website: None,
-                    security_contact: None,
-                    details: None,
-                },
                 power: 2,
             },
         ];
 
         // diff with itself must be empty
-        let (diff, update_members) = calculate_diff(vals.clone(), vals.clone());
+        let (diff, add, remove) = calculate_diff(vals.clone(), vals.clone());
         assert_eq!(diff.diffs, vec![]);
-        assert_eq!(update_members, update_members_msg(vec![], vec! {}));
+        assert_eq!((add, remove), (vec![], vec![]));
 
         // diff with empty must be itself (additions)
-        let (diff, update_members) = calculate_diff(vals.clone(), empty.clone());
+        let (diff, add, remove) = calculate_diff(vals.clone(), empty.clone());
         assert_eq!(
             vec![
                 ValidatorUpdate {
@@ -942,13 +1094,11 @@ mod test {
             ],
             diff.diffs
         );
-        assert_eq!(
-            update_members,
-            update_members_msg(vec![], vec![("op1", 1), ("op2", 2)])
-        );
+        assert!(remove.is_empty());
+        assert_eq!(add, members(vec![("op1", 1), ("op2", 2)]));
 
         // diff between empty and vals must be removals
-        let (diff, update_members) = calculate_diff(empty, vals.clone());
+        let (diff, add, remove) = calculate_diff(empty, vals.clone());
         assert_eq!(
             vec![
                 ValidatorUpdate {
@@ -962,28 +1112,19 @@ mod test {
             ],
             diff.diffs
         );
-        assert_eq!(
-            update_members,
-            update_members_msg(vec!["op1", "op2"], vec![])
-        );
+        assert!(add.is_empty());
+        assert_eq!(remove, ["op1", "op2"]);
 
         // Add a new member
         let mut cur = vals.clone();
         cur.push(ValidatorInfo {
             operator: Addr::unchecked("op3"),
             validator_pubkey: Pubkey::Ed25519(b"pubkey3".into()),
-            metadata: ValidatorMetadata {
-                moniker: "moniker3".to_owned(),
-                identity: None,
-                website: None,
-                security_contact: None,
-                details: None,
-            },
             power: 3,
         });
 
         // diff must be add last
-        let (diff, update_members) = calculate_diff(cur, vals.clone());
+        let (diff, add, remove) = calculate_diff(cur, vals.clone());
         assert_eq!(
             vec![ValidatorUpdate {
                 pubkey: Pubkey::Ed25519(b"pubkey3".into()),
@@ -991,13 +1132,14 @@ mod test {
             },],
             diff.diffs
         );
-        assert_eq!(update_members, update_members_msg(vec![], vec![("op3", 3)]));
+        assert!(remove.is_empty());
+        assert_eq!(add, members(vec![("op3", 3)]));
 
         // add all but (one) last member
         let old: Vec<_> = vals.iter().skip(1).cloned().collect();
 
         // diff must be add all but last
-        let (diff, update_members) = calculate_diff(vals.clone(), old);
+        let (diff, add, remove) = calculate_diff(vals.clone(), old);
         assert_eq!(
             vec![ValidatorUpdate {
                 pubkey: Pubkey::Ed25519(b"pubkey1".into()),
@@ -1005,12 +1147,13 @@ mod test {
             },],
             diff.diffs
         );
-        assert_eq!(update_members, update_members_msg(vec![], vec![("op1", 1)]));
+        assert!(remove.is_empty());
+        assert_eq!(add, members(vec![("op1", 1)]));
 
         // remove last member
         let cur: Vec<_> = vals.iter().take(1).cloned().collect();
         // diff must be remove last
-        let (diff, update_members) = calculate_diff(cur, vals.clone());
+        let (diff, add, remove) = calculate_diff(cur, vals.clone());
         assert_eq!(
             vec![ValidatorUpdate {
                 pubkey: Pubkey::Ed25519(b"pubkey2".into()),
@@ -1018,12 +1161,13 @@ mod test {
             },],
             diff.diffs
         );
-        assert_eq!(update_members, update_members_msg(vec!["op2"], vec![]));
+        assert!(add.is_empty());
+        assert_eq!(remove, ["op2"]);
 
         // remove all but last member
         let cur: Vec<_> = vals.iter().skip(1).cloned().collect();
         // diff must be remove all but last
-        let (diff, update_members) = calculate_diff(cur, vals);
+        let (diff, add, remove) = calculate_diff(cur, vals);
         assert_eq!(
             vec![ValidatorUpdate {
                 pubkey: Pubkey::Ed25519(b"pubkey1".into()),
@@ -1031,8 +1175,8 @@ mod test {
             },],
             diff.diffs
         );
-
-        assert_eq!(update_members, update_members_msg(vec!["op1"], vec![]));
+        assert!(add.is_empty());
+        assert_eq!(remove, ["op1"]);
     }
 
     // TODO: Another 7 in 1 test to be split
@@ -1042,12 +1186,13 @@ mod test {
         let vals = validators(VALIDATORS);
 
         // diff with itself must be empty
-        let (diff, update_members) = calculate_diff(vals.clone(), vals.clone());
+        let (diff, add, remove) = calculate_diff(vals.clone(), vals.clone());
         assert_eq!(diff.diffs, vec![]);
-        assert_eq!(update_members, update_members_msg(vec![], vec![]));
+        assert!(add.is_empty());
+        assert!(remove.is_empty());
 
         // diff with empty must be itself (additions)
-        let (diff, update_members) = calculate_diff(vals.clone(), empty.clone());
+        let (diff, add, remove) = calculate_diff(vals.clone(), empty.clone());
         assert_eq!(
             ValidatorDiff {
                 diffs: vals
@@ -1060,10 +1205,10 @@ mod test {
             },
             diff
         );
+        assert!(remove.is_empty());
         assert_eq!(
-            update_members,
-            update_members_msg(
-                vec![],
+            add,
+            members(
                 vals.iter()
                     .map(|vi| (vi.operator.as_str(), vi.power))
                     .collect()
@@ -1071,7 +1216,7 @@ mod test {
         );
 
         // diff between empty and vals must be removals
-        let (diff, update_members) = calculate_diff(empty, vals.clone());
+        let (diff, add, remove) = calculate_diff(empty, vals.clone());
         assert_eq!(
             ValidatorDiff {
                 diffs: vals
@@ -1084,16 +1229,19 @@ mod test {
             },
             diff
         );
+        assert!(add.is_empty());
         assert_eq!(
-            update_members,
-            update_members_msg(vals.iter().map(|vi| vi.operator.as_str()).collect(), vec![])
+            remove,
+            vals.iter()
+                .map(|vi| vi.operator.as_str())
+                .collect::<Vec<_>>()
         );
 
         // Add a new member
         let cur = validators(VALIDATORS + 1);
 
         // diff must be add last
-        let (diff, update_members) = calculate_diff(cur.clone(), vals.clone());
+        let (diff, add, remove) = calculate_diff(cur.clone(), vals.clone());
         assert_eq!(
             ValidatorDiff {
                 diffs: vec![ValidatorUpdate {
@@ -1103,22 +1251,20 @@ mod test {
             },
             diff
         );
+        assert!(remove.is_empty());
         assert_eq!(
-            update_members,
-            update_members_msg(
-                vec![],
-                vec![(
-                    cur.last().as_ref().unwrap().operator.as_str(),
-                    (VALIDATORS + 1) as u64
-                )]
-            )
+            add,
+            members(vec![(
+                cur.last().as_ref().unwrap().operator.as_str(),
+                (VALIDATORS + 1) as u64
+            )])
         );
 
         // add all but (one) last member
         let old: Vec<_> = vals.iter().skip(VALIDATORS - 1).cloned().collect();
 
         // diff must be add all but last
-        let (diff, update_members) = calculate_diff(vals.clone(), old);
+        let (diff, add, remove) = calculate_diff(vals.clone(), old);
         assert_eq!(
             ValidatorDiff {
                 diffs: vals
@@ -1132,10 +1278,10 @@ mod test {
             },
             diff
         );
+        assert!(remove.is_empty());
         assert_eq!(
-            update_members,
-            update_members_msg(
-                vec![],
+            add,
+            members(
                 vals.iter()
                     .take(VALIDATORS - 1)
                     .map(|vi| (vi.operator.as_ref(), vi.power))
@@ -1146,7 +1292,7 @@ mod test {
         // remove last member
         let cur: Vec<_> = vals.iter().take(VALIDATORS - 1).cloned().collect();
         // diff must be remove last
-        let (diff, update_members) = calculate_diff(cur, vals.clone());
+        let (diff, add, remove) = calculate_diff(cur, vals.clone());
         assert_eq!(
             ValidatorDiff {
                 diffs: vec![ValidatorUpdate {
@@ -1156,15 +1302,13 @@ mod test {
             },
             diff
         );
-        assert_eq!(
-            update_members,
-            update_members_msg(vec![vals.last().unwrap().operator.as_ref()], vec![])
-        );
+        assert!(add.is_empty());
+        assert_eq!(remove, vec![vals.last().unwrap().operator.as_ref()]);
 
         // remove all but last member
         let cur: Vec<_> = vals.iter().skip(VALIDATORS - 1).cloned().collect();
         // diff must be remove all but last
-        let (diff, update_members) = calculate_diff(cur, vals.clone());
+        let (diff, add, remove) = calculate_diff(cur, vals.clone());
         assert_eq!(
             ValidatorDiff {
                 diffs: vals
@@ -1178,15 +1322,13 @@ mod test {
             },
             diff
         );
+        assert!(add.is_empty());
         assert_eq!(
-            update_members,
-            update_members_msg(
-                vals.iter()
-                    .take(VALIDATORS - 1)
-                    .map(|vi| vi.operator.as_ref())
-                    .collect(),
-                vec![]
-            )
+            remove,
+            vals.iter()
+                .take(VALIDATORS - 1)
+                .map(|vi| vi.operator.as_ref())
+                .collect::<Vec<_>>()
         );
     }
 }
