@@ -18,13 +18,13 @@ pub struct InstantiateMsg {
     pub admin: Option<String>,
     /// Address of a cw4 contract with the raw membership used to feed the validator set
     pub membership: String,
-    /// Minimum weight needed by an address in `membership` to be considered for the validator set.
-    /// 0-weight members are always filtered out.
+    /// Minimum points needed by an address in `membership` to be considered for the validator set.
+    /// 0-point members are always filtered out.
     /// TODO: if we allow sub-1 scaling factors, determine if this is pre-/post- scaling
-    /// (use weight for cw4, power for Tendermint)
-    pub min_weight: u64,
+    /// (use points for cw4, power for Tendermint)
+    pub min_points: u64,
     /// The maximum number of validators that can be included in the Tendermint validator set.
-    /// If there are more validators than slots, we select the top N by membership weight
+    /// If there are more validators than slots, we select the top N by membership points
     /// descending. (In case of ties at the last slot, select by "first" Tendermint pubkey,
     /// lexicographically sorted).
     pub max_validators: u32,
@@ -44,8 +44,8 @@ pub struct InstantiateMsg {
     /// making this privileged/calling the EndBlockers, so that we have a non-empty validator set
     pub initial_keys: Vec<OperatorInitInfo>,
 
-    /// A scaling factor to multiply cw4-group weights to produce the Tendermint validator power
-    /// (TODO: should we allow this to reduce weight? Like 1/1000?)
+    /// A scaling factor to multiply cw4-group points to produce the Tendermint validator power
+    /// (TODO: should we allow this to reduce points? Like 1/1000?)
     pub scaling: Option<u32>,
 
     /// Percentage of total accumulated fees that is subtracted from tokens minted as rewards.
@@ -89,7 +89,125 @@ pub struct InstantiateMsg {
     /// ```
     ///
     /// This contract has to support all the `RewardsDistribution` messages
-    pub rewards_code_id: u64,
+    pub validator_group_code_id: u64,
+}
+
+impl InstantiateMsg {
+    pub fn validate(&self) -> Result<(), ContractError> {
+        if self.epoch_length == 0 {
+            return Err(ContractError::InvalidEpoch {});
+        }
+        if self.min_points == 0 {
+            return Err(ContractError::InvalidMinPoints {});
+        }
+        if self.max_validators == 0 {
+            return Err(ContractError::InvalidMaxValidators {});
+        }
+        if self.scaling == Some(0) {
+            return Err(ContractError::InvalidScaling {});
+        }
+        // Current denom regexp in the SDK is [a-zA-Z][a-zA-Z0-9/]{2,127}
+        if self.epoch_reward.denom.len() < 2 || self.epoch_reward.denom.len() > 127 {
+            return Err(ContractError::InvalidRewardDenom {});
+        }
+        for op in self.initial_keys.iter() {
+            op.validate()?
+        }
+        Ok(())
+    }
+}
+
+#[derive(Serialize, Deserialize, Clone, PartialEq, JsonSchema, Debug)]
+#[serde(rename_all = "snake_case")]
+pub enum ExecuteMsg {
+    /// Change the admin
+    UpdateAdmin {
+        admin: Option<String>,
+    },
+    /// Alter config values
+    UpdateConfig {
+        min_points: Option<u64>,
+        max_validators: Option<u32>,
+    },
+    /// Links info.sender (operator) to this Tendermint consensus key.
+    /// The operator cannot re-register another key.
+    /// No two operators may have the same consensus_key.
+    RegisterValidatorKey {
+        pubkey: Pubkey,
+        /// Additional metadata assigned to this validator
+        metadata: ValidatorMetadata,
+    },
+    UpdateMetadata(ValidatorMetadata),
+    /// Jails validator. Can be executed only by the admin.
+    Jail {
+        /// Operator which should be jailed
+        operator: String,
+        /// Duration for how long validator is jailed
+        duration: JailingDuration,
+    },
+    /// Unjails validator. Admin can unjail anyone anytime, others can unjail only themselves and
+    /// only if the jail period passed.
+    Unjail {
+        /// Address to unjail. Optional, as if not provided it is assumed to be the sender of the
+        /// message (for convenience when unjailing self after the jail period).
+        operator: Option<String>,
+    },
+    /// To be called by admin only. Slashes a given address (by forwarding slash to both rewards
+    /// contract and engagement contract)
+    Slash {
+        addr: String,
+        portion: Decimal,
+    },
+
+    /// This will update the validator set with the passed list.
+    /// Used for testing validators storage.
+    #[cfg(debug_assertions)]
+    SimulateValidatorSet {
+        validators: Vec<ValidatorInfo>,
+    },
+}
+
+#[derive(Serialize, Deserialize, Clone, PartialEq, JsonSchema, Debug)]
+#[serde(rename_all = "snake_case")]
+pub enum QueryMsg {
+    /// Returns configuration
+    Configuration {},
+    /// Returns EpochResponse - get info on current and next epochs
+    Epoch {},
+
+    /// Returns the validator key and associated metadata (if present) for the given operator.
+    /// Returns ValidatorResponse
+    Validator { operator: String },
+    /// Paginate over all operators, using operator address as pagination.
+    /// Returns ListValidatorsResponse
+    ListValidators {
+        start_after: Option<String>,
+        limit: Option<u32>,
+    },
+
+    /// List the current validator set, sorted by power descending
+    ListActiveValidators {
+        start_after: Option<String>,
+        limit: Option<u32>,
+    },
+
+    /// Returns ListValidatorsResponse
+    ListJailedValidators {
+        start_after: Option<String>,
+        limit: Option<u32>,
+    },
+
+    /// This will calculate who the new validators would be if
+    /// we recalculated end block right now.
+    /// Also returns ListActiveValidatorsResponse
+    SimulateActiveValidators {},
+
+    /// Returns a list of validator slashing events.
+    /// Returns ListValidatorSlashingResponse
+    ListValidatorSlashing { operator: String },
+
+    /// Returns cw_controllers::AdminResponse
+    Admin {},
 }
 
 #[derive(Serialize, Deserialize, Clone, PartialEq, JsonSchema, Debug)]
@@ -146,31 +264,6 @@ pub fn default_double_sign_slash() -> Decimal {
     Decimal::percent(50)
 }
 
-impl InstantiateMsg {
-    pub fn validate(&self) -> Result<(), ContractError> {
-        if self.epoch_length == 0 {
-            return Err(ContractError::InvalidEpoch {});
-        }
-        if self.min_weight == 0 {
-            return Err(ContractError::InvalidMinWeight {});
-        }
-        if self.max_validators == 0 {
-            return Err(ContractError::InvalidMaxValidators {});
-        }
-        if self.scaling == Some(0) {
-            return Err(ContractError::InvalidScaling {});
-        }
-        // Current denom regexp in the SDK is [a-zA-Z][a-zA-Z0-9/]{2,127}
-        if self.epoch_reward.denom.len() < 2 || self.epoch_reward.denom.len() > 127 {
-            return Err(ContractError::InvalidRewardDenom {});
-        }
-        for op in self.initial_keys.iter() {
-            op.validate()?
-        }
-        Ok(())
-    }
-}
-
 /// Validator Metadata modeled after the Cosmos SDK staking module
 #[derive(
     Serialize, Deserialize, Clone, Eq, PartialEq, Ord, PartialOrd, JsonSchema, Debug, Default,
@@ -192,12 +285,56 @@ pub struct ValidatorMetadata {
     pub details: Option<String>,
 }
 
-const MIN_MONIKER_LENGTH: usize = 3;
+pub const MIN_MONIKER_LENGTH: usize = 3;
+pub const MIN_METADATA_SIZE: usize = 1;
+pub const MAX_METADATA_SIZE: usize = 256;
 
 impl ValidatorMetadata {
     pub fn validate(&self) -> Result<(), ContractError> {
-        if self.moniker.len() < MIN_MONIKER_LENGTH {
-            return Err(ContractError::InvalidMoniker {});
+        if self.moniker.len() < MIN_MONIKER_LENGTH || self.moniker.len() > MAX_METADATA_SIZE {
+            return Err(ContractError::InvalidMetadata {
+                data: "moniker",
+                min: MIN_MONIKER_LENGTH,
+                max: MAX_METADATA_SIZE,
+            });
+        }
+        if let Some(identity) = &self.identity {
+            if identity.is_empty() || identity.len() > MAX_METADATA_SIZE {
+                return Err(ContractError::InvalidMetadata {
+                    data: "identity",
+                    min: MIN_METADATA_SIZE,
+                    max: MAX_METADATA_SIZE,
+                });
+            }
+        }
+        if let Some(website) = &self.website {
+            if website.is_empty() || website.len() > MAX_METADATA_SIZE {
+                return Err(ContractError::InvalidMetadata {
+                    data: "website",
+                    min: MIN_METADATA_SIZE,
+                    max: MAX_METADATA_SIZE,
+                });
+            } else if !website.starts_with("https://") && !website.starts_with("http://") {
+                return Err(ContractError::InvalidMetadataWebsitePrefix {});
+            }
+        }
+        if let Some(security_contract) = &self.security_contact {
+            if security_contract.is_empty() || security_contract.len() > MAX_METADATA_SIZE {
+                return Err(ContractError::InvalidMetadata {
+                    data: "security_contract",
+                    min: MIN_METADATA_SIZE,
+                    max: MAX_METADATA_SIZE,
+                });
+            }
+        }
+        if let Some(details) = &self.details {
+            if details.is_empty() || details.len() > MAX_METADATA_SIZE {
+                return Err(ContractError::InvalidMetadata {
+                    data: "details",
+                    min: MIN_METADATA_SIZE,
+                    max: MAX_METADATA_SIZE,
+                });
+            }
         }
         Ok(())
     }
@@ -219,86 +356,6 @@ impl OperatorInitInfo {
     }
 }
 
-#[derive(Serialize, Deserialize, Clone, PartialEq, JsonSchema, Debug)]
-#[serde(rename_all = "snake_case")]
-pub enum ExecuteMsg {
-    /// Change the admin
-    UpdateAdmin {
-        admin: Option<String>,
-    },
-    /// Alter config values
-    UpdateConfig {
-        min_weight: Option<u64>,
-        max_validators: Option<u32>,
-    },
-    /// Links info.sender (operator) to this Tendermint consensus key.
-    /// The operator cannot re-register another key.
-    /// No two operators may have the same consensus_key.
-    RegisterValidatorKey {
-        pubkey: Pubkey,
-        /// Additional metadata assigned to this validator
-        metadata: ValidatorMetadata,
-    },
-    UpdateMetadata(ValidatorMetadata),
-    /// Jails validator. Can be executed only by the admin.
-    Jail {
-        /// Operator which should be jailed
-        operator: String,
-        /// Duration for how long validator is jailed
-        duration: JailingDuration,
-    },
-    /// Unjails validator. Admin can unjail anyone anytime, others can unjail only themselves and
-    /// only if the jail period passed.
-    Unjail {
-        /// Address to unjail. Optional, as if not provided it is assumed to be the sender of the
-        /// message (for convenience when unjailing self after the jail period).
-        operator: Option<String>,
-    },
-    /// To be called by admin only. Slashes a given address (by forwarding slash to both rewards
-    /// contract and engagement contract)
-    Slash {
-        addr: String,
-        portion: Decimal,
-    },
-}
-
-#[derive(Serialize, Deserialize, Clone, PartialEq, JsonSchema, Debug)]
-#[serde(rename_all = "snake_case")]
-pub enum QueryMsg {
-    /// Returns configuration
-    Configuration {},
-    /// Returns EpochResponse - get info on current and next epochs
-    Epoch {},
-
-    /// Returns the validator key and associated metadata (if present) for the given operator.
-    /// Returns ValidatorResponse
-    Validator { operator: String },
-    /// Paginate over all operators, using operator address as pagination.
-    /// Returns ListValidatorsResponse
-    ListValidators {
-        start_after: Option<String>,
-        limit: Option<u32>,
-    },
-
-    /// List the current validator set, sorted by power descending
-    ListActiveValidators {
-        start_after: Option<String>,
-        limit: Option<u32>,
-    },
-
-    /// This will calculate who the new validators would be if
-    /// we recalculated end block right now.
-    /// Also returns ListActiveValidatorsResponse
-    SimulateActiveValidators {},
-
-    /// Returns a list of validator slashing events.
-    /// Returns ListValidatorSlashingResponse
-    ListValidatorSlashing { operator: String },
-
-    /// Returns cw_controllers::AdminResponse
-    Admin {},
-}
-
 #[derive(Serialize, Deserialize, Clone, PartialEq, JsonSchema, Debug)]
 pub struct EpochResponse {
     /// Number of seconds in one epoch. We update the Tendermint validator set only once per epoch.
@@ -339,9 +396,10 @@ impl OperatorResponse {
 }
 
 #[derive(Serialize, Deserialize, Clone, PartialEq, JsonSchema, Debug)]
+#[serde(rename_all = "snake_case")]
 pub enum JailingPeriod {
-    Forever {},
     Until(Expiration),
+    Forever {},
 }
 
 impl JailingPeriod {
@@ -397,7 +455,7 @@ pub struct ListValidatorSlashingResponse {
 pub enum DistributionMsg {
     /// Message sent to `distribution_contract` with funds which are part of the reward to be split
     /// between engaged operators
-    DistributeFunds {},
+    DistributeRewards {},
 }
 
 #[derive(Serialize, Deserialize, Clone, PartialEq, JsonSchema, Debug)]
@@ -415,19 +473,19 @@ pub enum RewardsDistribution {
         remove: Vec<String>,
         add: Vec<Member>,
     },
-    DistributeFunds {},
+    DistributeRewards {},
 }
 
 #[derive(Serialize, Deserialize, Clone, PartialEq, JsonSchema, Debug)]
 #[serde(rename_all = "snake_case")]
 pub struct InstantiateResponse {
-    pub rewards_contract: Addr,
+    pub validator_group: Addr,
 }
 
 #[derive(Serialize, Deserialize, Clone, PartialEq, JsonSchema, Debug)]
 #[serde(rename_all = "snake_case")]
 pub struct MigrateMsg {
-    pub min_weight: Option<u64>,
+    pub min_points: Option<u64>,
     pub max_validators: Option<u32>,
 }
 
@@ -450,7 +508,7 @@ mod test {
         let proper = InstantiateMsg {
             admin: None,
             membership: "contract-addr".into(),
-            min_weight: 5,
+            min_points: 5,
             max_validators: 20,
             epoch_length: 5000,
             epoch_reward: coin(7777, "foobar"),
@@ -460,7 +518,7 @@ mod test {
             auto_unjail: false,
             double_sign_slash_ratio: Decimal::percent(50),
             distribution_contracts: UnvalidatedDistributionContracts::default(),
-            rewards_code_id: 0,
+            validator_group_code_id: 0,
         };
         proper.validate().unwrap();
 
@@ -475,11 +533,11 @@ mod test {
         let err = invalid.validate().unwrap_err();
         assert_eq!(err, ContractError::InvalidScaling {});
 
-        // fails on 0 min weight
+        // fails on 0 min points
         let mut invalid = proper.clone();
-        invalid.min_weight = 0;
+        invalid.min_points = 0;
         let err = invalid.validate().unwrap_err();
-        assert_eq!(err, ContractError::InvalidMinWeight {});
+        assert_eq!(err, ContractError::InvalidMinPoints {});
 
         // fails on 0 max validators
         let mut invalid = proper.clone();
@@ -510,4 +568,132 @@ mod test {
         let err = invalid.validate().unwrap_err();
         assert_eq!(err, ContractError::InvalidRewardDenom {});
     }
+
+    #[test]
+    fn validate_metadata() {
+        let meta = ValidatorMetadata {
+            moniker: "example".to_owned(),
+            identity: Some((0..MAX_METADATA_SIZE + 1).map(|_| "X").collect::<String>()),
+            website: Some((0..MAX_METADATA_SIZE + 1).map(|_| "X").collect::<String>()),
+            security_contact: Some((0..MAX_METADATA_SIZE + 1).map(|_| "X").collect::<String>()),
+            details: Some((0..MAX_METADATA_SIZE + 1).map(|_| "X").collect::<String>()),
+        };
+        let resp = meta.validate().unwrap_err();
+        assert_eq!(
+            ContractError::InvalidMetadata {
+                data: "identity",
+                min: MIN_METADATA_SIZE,
+                max: MAX_METADATA_SIZE
+            },
+            resp
+        );
+
+        let meta = ValidatorMetadata {
+            identity: Some("identity".to_owned()),
+            ..meta
+        };
+        let resp = meta.validate().unwrap_err();
+        assert_eq!(
+            ContractError::InvalidMetadata {
+                data: "website",
+                min: MIN_METADATA_SIZE,
+                max: MAX_METADATA_SIZE,
+            },
+            resp
+        );
+
+        let meta = ValidatorMetadata {
+            website: Some("https://website".to_owned()),
+            ..meta
+        };
+        let resp = meta.validate().unwrap_err();
+        assert_eq!(
+            ContractError::InvalidMetadata {
+                data: "security_contract",
+                min: MIN_METADATA_SIZE,
+                max: MAX_METADATA_SIZE,
+            },
+            resp
+        );
+
+        let meta = ValidatorMetadata {
+            security_contact: Some("contract".to_owned()),
+            ..meta
+        };
+        let resp = meta.validate().unwrap_err();
+        assert_eq!(
+            ContractError::InvalidMetadata {
+                data: "details",
+                min: MIN_METADATA_SIZE,
+                max: MAX_METADATA_SIZE,
+            },
+            resp
+        );
+
+        let meta = ValidatorMetadata {
+            identity: Some(String::new()),
+            website: Some(String::new()),
+            security_contact: Some(String::new()),
+            details: Some(String::new()),
+            ..meta
+        };
+        let resp = meta.validate().unwrap_err();
+        assert_eq!(
+            ContractError::InvalidMetadata {
+                data: "identity",
+                min: MIN_METADATA_SIZE,
+                max: MAX_METADATA_SIZE
+            },
+            resp
+        );
+
+        let meta = ValidatorMetadata {
+            identity: Some("identity".to_owned()),
+            ..meta
+        };
+        let resp = meta.validate().unwrap_err();
+        assert_eq!(
+            ContractError::InvalidMetadata {
+                data: "website",
+                min: MIN_METADATA_SIZE,
+                max: MAX_METADATA_SIZE,
+            },
+            resp
+        );
+
+        let meta = ValidatorMetadata {
+            website: Some("http://website".to_owned()),
+            ..meta
+        };
+        let resp = meta.validate().unwrap_err();
+        assert_eq!(
+            ContractError::InvalidMetadata {
+                data: "security_contract",
+                min: MIN_METADATA_SIZE,
+                max: MAX_METADATA_SIZE,
+            },
+            resp
+        );
+
+        let meta = ValidatorMetadata {
+            security_contact: Some("contract".to_owned()),
+            ..meta
+        };
+        let resp = meta.validate().unwrap_err();
+        assert_eq!(
+            ContractError::InvalidMetadata {
+                data: "details",
+                min: MIN_METADATA_SIZE,
+                max: MAX_METADATA_SIZE,
+            },
+            resp
+        );
+
+        let meta = ValidatorMetadata {
+            website: Some("website".to_owned()),
+            ..meta
+        };
+        let resp = meta.validate().unwrap_err();
+        assert_eq!(ContractError::InvalidMetadataWebsitePrefix {}, resp);
+    }
 }
