@@ -3,7 +3,9 @@ mod contract;
 mod double_sign;
 mod helpers;
 mod jailing;
+mod migration;
 mod rewards_split;
 mod slashing;
 mod stake;
 mod suite;
+mod update_config;
