@@ -1,5 +1,6 @@
-use super::helpers::{addr_to_pubkey, mock_metadata, mock_pubkey};
+use super::helpers::addr_to_pubkey;
 use crate::state::Config;
+use crate::test_helpers::{mock_metadata, mock_pubkey};
 use crate::{msg::*, state::ValidatorInfo};
 use anyhow::{bail, Result as AnyResult};
 use cosmwasm_std::{
@@ -23,7 +24,7 @@ pub fn contract_engagement() -> Box<dyn Contract<TgradeMsg>> {
     Box::new(contract)
 }
 
-fn contract_stake() -> Box<dyn Contract<TgradeMsg>> {
+pub fn contract_stake() -> Box<dyn Contract<TgradeMsg>> {
     let contract = ContractWrapper::new(
         tg4_stake::contract::execute,
         tg4_stake::contract::instantiate,
@@ -64,7 +65,7 @@ enum GroupConfig {
     /// Usa a tg4_stake contract as the valset group contract
     Stake {
         denom: String,
-        tokens_per_weight: Uint128,
+        tokens_per_points: Uint128,
     },
 }
 
@@ -77,7 +78,7 @@ pub struct SuiteBuilder {
     /// Valset operators, with optionally provided pubkeys
     operators: Vec<(String, Option<Pubkey>)>,
     #[derivative(Default(value = "1"))]
-    min_weight: u64,
+    min_points: u64,
     /// Maximum number of validators for single epoch
     #[derivative(Default(value = "u32::MAX"))]
     max_validators: u32,
@@ -87,7 +88,7 @@ pub struct SuiteBuilder {
     /// Base epoch reward, 100uscd by default
     #[derivative(Default(value = "coin(100, \"usdc\")"))]
     epoch_reward: Coin,
-    /// Validators weight scaling
+    /// Validators points scaling
     scaling: Option<u32>,
     /// Factor determining how accumulated fees affects base epoch reward
     fee_percentage: Decimal,
@@ -123,12 +124,12 @@ impl SuiteBuilder {
     }
 
     /// Use a tg4_engagement contract for membership.
-    /// Provide a list of members with assigned weights to initialize it with.
+    /// Provide a list of members with assigned points to initialize it with.
     pub fn with_engagement(mut self, members: &[(&str, u64)]) -> Self {
         self.group_config = GroupConfig::Engagement {
             members: members
                 .iter()
-                .map(|(addr, weight)| ((*addr).to_owned(), *weight))
+                .map(|(addr, points)| ((*addr).to_owned(), *points))
                 .collect(),
         };
 
@@ -140,11 +141,11 @@ impl SuiteBuilder {
     pub fn with_stake(
         mut self,
         denom: impl Into<String>,
-        tokens_per_weight: impl Into<Uint128>,
+        tokens_per_points: impl Into<Uint128>,
     ) -> Self {
         self.group_config = GroupConfig::Stake {
             denom: denom.into(),
-            tokens_per_weight: tokens_per_weight.into(),
+            tokens_per_points: tokens_per_points.into(),
         };
         self
     }
@@ -177,9 +178,9 @@ impl SuiteBuilder {
         let config = DistributionConfig {
             members: members
                 .iter()
-                .map(|(addr, weight)| Member {
+                .map(|(addr, points)| Member {
                     addr: (*addr).to_owned(),
-                    weight: *weight,
+                    points: *points,
                 })
                 .collect(),
             halflife: halflife.into(),
@@ -199,8 +200,8 @@ impl SuiteBuilder {
         self
     }
 
-    pub fn with_min_weight(mut self, min_weight: u64) -> Self {
-        self.min_weight = min_weight;
+    pub fn with_min_points(mut self, min_points: u64) -> Self {
+        self.min_points = min_points;
         self
     }
 
@@ -226,7 +227,7 @@ impl SuiteBuilder {
 
                 let members = members
                     .into_iter()
-                    .map(|(addr, weight)| Member { addr, weight })
+                    .map(|(addr, points)| Member { addr, points })
                     .collect();
 
                 app.instantiate_contract(
@@ -248,7 +249,7 @@ impl SuiteBuilder {
             }
             GroupConfig::Stake {
                 denom,
-                tokens_per_weight,
+                tokens_per_points,
             } => {
                 let stake_id = app.store_code(contract_stake());
                 app.instantiate_contract(
@@ -256,7 +257,7 @@ impl SuiteBuilder {
                     admin.clone(),
                     &tg4_stake::msg::InstantiateMsg {
                         denom,
-                        tokens_per_weight,
+                        tokens_per_point: tokens_per_points,
                         min_bond: Uint128::zero(),
                         unbonding_period: 0,
                         admin: Some(admin.to_string()),
@@ -333,7 +334,7 @@ impl SuiteBuilder {
                 &InstantiateMsg {
                     admin: Some(admin.to_string()),
                     membership: membership.to_string(),
-                    min_weight: self.min_weight,
+                    min_points: self.min_points,
                     max_validators: self.max_validators,
                     epoch_length: self.epoch_length,
                     epoch_reward: self.epoch_reward,
@@ -345,7 +346,7 @@ impl SuiteBuilder {
                     distribution_contracts: UnvalidatedDistributionContracts {
                         inner: distribution_contract_instantiation_info,
                     },
-                    rewards_code_id: engagement_id,
+                    validator_group_code_id: engagement_id,
                 },
                 &[],
                 "valset",
@@ -400,7 +401,7 @@ impl SuiteBuilder {
             operators: operators.into_iter().map(|o| o.operator).collect(),
             epoch_length: self.epoch_length,
             denom,
-            rewards_contract: resp.rewards_contract,
+            validator_group: resp.validator_group,
         }
     }
 }
@@ -428,7 +429,7 @@ pub struct Suite {
     /// Reward denom
     denom: String,
     /// Rewards distribution contract address
-    rewards_contract: Addr,
+    validator_group: Addr,
 }
 
 impl Suite {
@@ -559,14 +560,14 @@ impl Suite {
     pub fn update_config(
         &mut self,
         executor: &str,
-        min_weight: impl Into<Option<u64>>,
+        min_points: impl Into<Option<u64>>,
         max_validators: impl Into<Option<u32>>,
     ) -> AnyResult<AppResponse> {
         self.app.execute_contract(
             Addr::unchecked(executor),
             self.valset.clone(),
             &ExecuteMsg::UpdateConfig {
-                min_weight: min_weight.into(),
+                min_points: min_points.into(),
                 max_validators: max_validators.into(),
             },
             &[],
@@ -599,7 +600,7 @@ impl Suite {
             self.app.execute_contract(
                 Addr::unchecked(executor),
                 contract.clone(),
-                &tg4_engagement::msg::ExecuteMsg::WithdrawFunds {
+                &tg4_engagement::msg::ExecuteMsg::WithdrawRewards {
                     owner: None,
                     receiver: None,
                 },
@@ -616,8 +617,8 @@ impl Suite {
     pub fn withdraw_validation_reward(&mut self, executor: &str) -> AnyResult<AppResponse> {
         self.app.execute_contract(
             Addr::unchecked(executor),
-            self.rewards_contract.clone(),
-            &tg4_engagement::msg::ExecuteMsg::WithdrawFunds {
+            self.validator_group.clone(),
+            &tg4_engagement::msg::ExecuteMsg::WithdrawRewards {
                 owner: None,
                 receiver: None,
             },
@@ -687,6 +688,22 @@ impl Suite {
         Ok(resp.validators)
     }
 
+    pub fn list_jailed_validators(
+        &self,
+        start_after: impl Into<Option<String>>,
+        limit: impl Into<Option<u32>>,
+    ) -> StdResult<Vec<OperatorResponse>> {
+        let resp: ListValidatorResponse = self.app.wrap().query_wasm_smart(
+            self.valset.clone(),
+            &QueryMsg::ListJailedValidators {
+                start_after: start_after.into(),
+                limit: limit.into(),
+            },
+        )?;
+
+        Ok(resp.validators)
+    }
+
     pub fn list_validator_slashing(&self, addr: &str) -> StdResult<ListValidatorSlashingResponse> {
         let resp = self.app.wrap().query_wasm_smart(
             self.valset.clone(),
