@@ -1,17 +1,9 @@
 use crate::msg::{DistributionMsg, RewardsDistribution};
 use crate::state::Config;
-use cosmwasm_std::{
-    coins, to_binary, Addr, Coin, DepsMut, Env, StdResult, SubMsg, Uint128, WasmMsg,
-};
+use cosmwasm_std::{coins, to_binary, Coin, DepsMut, Env, StdResult, SubMsg, Uint128, WasmMsg};
 use tg_bindings::TgradeMsg;
 
-#[derive(Clone)]
-pub struct DistributionInfo {
-    pub addr: Addr,
-    pub weight: u64,
-}
-
-/// Ensure you pass in non-empty pay-validators, it will panic if total validator weight is 0
+/// Ensure you pass in non-empty pay-validators, it will panic if total validator points is 0
 /// This handles all deps and calls into pure functions
 pub fn pay_block_rewards(
     deps: DepsMut,
@@ -54,7 +46,7 @@ pub fn pay_block_rewards(
             reward_pool -= reward;
             messages.push(SubMsg::new(WasmMsg::Execute {
                 contract_addr: contract.contract.to_string(),
-                msg: to_binary(&DistributionMsg::DistributeFunds {})?,
+                msg: to_binary(&DistributionMsg::DistributeRewards {})?,
                 funds: coins(reward.into(), &block_reward.denom),
             }));
         }
@@ -63,8 +55,8 @@ pub fn pay_block_rewards(
     // After rewarding all non-validators, the remainder goes to validators.
     if reward_pool > Uint128::zero() {
         messages.push(SubMsg::new(WasmMsg::Execute {
-            contract_addr: config.rewards_contract.to_string(),
-            msg: to_binary(&RewardsDistribution::DistributeFunds {})?,
+            contract_addr: config.validator_group.to_string(),
+            msg: to_binary(&RewardsDistribution::DistributeRewards {})?,
             funds: coins(reward_pool.into(), &block_reward.denom),
         }));
     }
