@@ -12,26 +12,26 @@ use tg_bindings::{Ed25519Pubkey, Pubkey};
 pub struct Config {
     /// address of a tg4 contract with the raw membership used to feed the validator set
     pub membership: Tg4Contract,
-    /// minimum weight needed by an address in `membership` to be considered for the validator set.
-    /// 0-weight members are always filtered out.
+    /// minimum points needed by an address in `membership` to be considered for the validator set.
+    /// 0-point members are always filtered out.
     /// TODO: if we allow sub-1 scaling factors, determine if this is pre-/post- scaling
-    /// (use weight for tg4, power for tendermint)
-    pub min_weight: u64,
+    /// (use points for tg4, power for tendermint)
+    pub min_points: u64,
     /// The maximum number of validators that can be included in the Tendermint validator set.
-    /// If there are more validators than slots, we select the top N by membership weight
+    /// If there are more validators than slots, we select the top N by membership points
     /// descending. (In case of ties at the last slot, select by "first" tendermint pubkey
     /// lexicographically sorted).
     pub max_validators: u32,
-    /// A scaling factor to multiply tg4-engagement weights to produce the tendermint validator power
-    /// (TODO: should we allow this to reduce weight? Like 1/1000?)
+    /// A scaling factor to multiply tg4-engagement points to produce the tendermint validator power
+    /// (TODO: should we allow this to reduce points? Like 1/1000?)
     pub scaling: Option<u32>,
     /// Total reward paid out each epoch. This will be split among all validators during the last
     /// epoch.
     /// (epoch_reward.amount * 86_400 * 30 / epoch_length) is reward tokens to mint each month.
     /// Ensure this is sensible in relation to the total token supply.
     pub epoch_reward: Coin,
-    /// Percentage of total accumulated fees which is substracted from tokens minted as a rewards.
-    /// 50% as default. To disable this feature just set it to 0 (which efectivelly means that fees
+    /// Percentage of total accumulated fees which is subtracted from tokens minted as a rewards.
+    /// 50% as default. To disable this feature just set it to 0 (which effectively means that fees
     /// doesn't affect the per epoch reward).
     #[serde(default = "default_fee_percentage")]
     pub fee_percentage: Decimal,
@@ -51,8 +51,8 @@ pub struct Config {
     /// rewards contract.
     pub distribution_contracts: Vec<DistributionContract>,
 
-    /// Address of contract for rewards distribution.
-    pub rewards_contract: Addr,
+    /// Address of contract for validator group voting.
+    pub validator_group: Addr,
 }
 
 #[derive(Serialize, Deserialize, Clone, PartialEq, JsonSchema, Debug)]
@@ -80,7 +80,6 @@ pub struct ValidatorInfo {
     /// TODO: better name to specify this is the Tendermint pubkey for consensus?
     pub validator_pubkey: Pubkey,
     pub operator: Addr,
-    pub metadata: ValidatorMetadata,
     /// The voting power in Tendermint sdk
     pub power: u64,
 }
@@ -103,13 +102,13 @@ pub const VALIDATOR_SLASHING: Map<&Addr, Vec<ValidatorSlashing>> = Map::new("val
 /// is not jailed
 pub const JAIL: Map<&Addr, JailingPeriod> = Map::new("jail");
 
-/// This stores the immutable info for an operator. Both their Tendermint key as well as
+/// This stores the info for an operator. Both their Tendermint key as well as
 /// their metadata.
 #[derive(Serialize, Deserialize, Clone, JsonSchema, Debug, PartialEq)]
 pub struct OperatorInfo {
     pub pubkey: Ed25519Pubkey,
     pub metadata: ValidatorMetadata,
-    /// Is this an active validator?
+    /// Is this currently an active validator?
     pub active_validator: bool,
 }
 
